-- ****************************************************************************
-- * Licensed Materials - Property of IBM
-- * 5724-T40 IBM License Metric Tool
-- * 5724-S94 IBM Tivoli Asset Discovery for Distributed
-- *
-- * (C) Copyright IBM Corp. 2012.  All Rights Reserved.
-- *
-- * US Government Users Restricted Rights - Use, duplication or
-- * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
-- ****************************************************************************

create procedure ADM.DROP_PROCEDURE_IF_EXISTS(in PROCEDURE_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP PROCEDURE ' concat PROCEDURE_NAME;
	prepare S1 from STMT;
	execute S1;
	end%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.DROP_TABLE_IF_EXISTS')%	
	
create procedure ADM.DROP_TABLE_IF_EXISTS(in TABLE_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP TABLE ' concat TABLE_NAME;
	prepare S1 from STMT;
	execute S1;
	end%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.TRANSACTION_COMMIT')%	
	
create procedure ADM.TRANSACTION_COMMIT(in SQL_QUERY varchar(1000))
	begin
	declare STMT varchar(1000);
	DECLARE numrows INT DEFAULT 0;

	prepare S1 from SQL_QUERY;
	
	loop_statement:
	REPEAT
		execute S1;
		GET DIAGNOSTICS numrows = ROW_COUNT;
		COMMIT;
		UNTIL numrows = 0 
	END REPEAT loop_statement;
	end%
	
call ADM.TRANSACTION_COMMIT('
	delete from (select 1 from adm.AGT_VM_REL where START_TIME = END_TIME fetch first 20000 rows only)
')%	
	
call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.REMOVE_DUPLICATES')%

CREATE procedure ADM.REMOVE_DUPLICATES (  ) 
  LANGUAGE SQL
  BEGIN
	DECLARE V_AGENT_ID BIGINT;
	DECLARE V_VM_ID BIGINT;
	DECLARE V_START_TIME TIMESTAMP;
	DECLARE V_END_TIME TIMESTAMP;
	DECLARE V_LAST_MODIFIED TIMESTAMP;
  DECLARE v_numRecords INTEGER;
  DECLARE v_counter INTEGER DEFAULT 0;
  DECLARE c1 CURSOR WITH HOLD FOR 
      select 
        rel.AGENT_ID, 
        rel.VM_ID, 
        rel.START_TIME, 
        rel.END_TIME,
        rel.LAST_MODIFIED
      from adm.AGT_VM_REL rel
      left join adm.vm v 
        on v.id = rel.VM_ID
      where v.id is null;
  commit;
  SELECT COUNT(*) INTO v_numRecords 
      from adm.AGT_VM_REL rel
      left join adm.vm v 
        on v.id = rel.VM_ID
      where v.id is null; 
  OPEN c1;
  WHILE (v_counter < v_numRecords) DO	
    FETCH c1 INTO V_AGENT_ID, V_VM_ID, V_START_TIME, V_END_TIME, V_LAST_MODIFIED;
    delete from adm.AGT_VM_REL where AGENT_ID = V_AGENT_ID and VM_ID = V_VM_ID and START_TIME = V_START_TIME 
      and END_TIME = V_END_TIME and LAST_MODIFIED = V_LAST_MODIFIED;
    IF MOD(v_counter, 1000) = 0 THEN
      commit;
    END IF;
   SET v_counter = v_counter + 1;
  END WHILE;
  CLOSE c1; 
  commit;
  end
%

call ADM.REMOVE_DUPLICATES ()%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.REMOVE_DUPLICATES')%

CREATE procedure ADM.REMOVE_DUPLICATES (  ) 
  LANGUAGE SQL
  BEGIN
	DECLARE V_AGENT_ID BIGINT;
	DECLARE V_VM_ID BIGINT;
	DECLARE V_START_TIME TIMESTAMP;
	DECLARE V_END_TIME TIMESTAMP;
	DECLARE V_LAST_MODIFIED TIMESTAMP;
  DECLARE v_numRecords INTEGER;
  DECLARE v_counter INTEGER DEFAULT 0;
  DECLARE c1 CURSOR WITH HOLD FOR 
      select 
        rel.AGENT_ID, 
        rel.VM_ID, 
        rel.START_TIME, 
        rel.END_TIME,
        rel.LAST_MODIFIED
      from 
      (
          select 
              AGENT_ID, 
              VM_ID, 
              START_TIME, 
              END_TIME, 
              LAST_MODIFIED, 
              row_number() over ( PARTITION BY AGENT_ID, START_TIME) as nr 
          from adm.AGT_VM_REL
      ) as rel
      where rel.nr > 1;
  commit;
  select count(*) INTO v_numRecords 
      from
      (
          select 
              AGENT_ID, 
              VM_ID, 
              START_TIME, 
              END_TIME, 
              LAST_MODIFIED, 
              row_number() over ( PARTITION BY AGENT_ID, START_TIME) as nr 
          from adm.AGT_VM_REL
      ) as rel
      where rel.nr > 1;
  OPEN c1;
  WHILE (v_counter < v_numRecords) DO	
    FETCH c1 INTO V_AGENT_ID, V_VM_ID, V_START_TIME, V_END_TIME, V_LAST_MODIFIED;
    delete from adm.AGT_VM_REL where AGENT_ID = V_AGENT_ID and VM_ID = V_VM_ID and START_TIME = V_START_TIME 
      and END_TIME = V_END_TIME and LAST_MODIFIED = V_LAST_MODIFIED;
    IF MOD(v_counter, 1000) = 0 THEN
      commit;
    END IF;
   SET v_counter = v_counter + 1;
  END WHILE;
  CLOSE c1; 
  commit;
  end
%

call ADM.REMOVE_DUPLICATES ()%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.REMOVE_DUPLICATES')%

CREATE procedure ADM.REMOVE_DUPLICATES (  ) 
  LANGUAGE SQL
  BEGIN
	DECLARE V_AGENT_ID BIGINT;
	DECLARE V_VM_ID BIGINT;
	DECLARE V_START_TIME TIMESTAMP;
	DECLARE V_END_TIME TIMESTAMP;
	DECLARE V_LAST_MODIFIED TIMESTAMP;
  DECLARE v_numRecords INTEGER;
  DECLARE v_counter INTEGER DEFAULT 0;
  DECLARE c1 CURSOR WITH HOLD FOR 
      select 
        rel.AGENT_ID, 
        rel.VM_ID, 
        rel.START_TIME, 
        rel.END_TIME,
        rel.LAST_MODIFIED
      from 
      (
          select 
              AGENT_ID, 
              VM_ID, 
              START_TIME, 
              END_TIME, 
              LAST_MODIFIED, 
              row_number() over ( PARTITION BY AGENT_ID, END_TIME) as nr 
          from adm.AGT_VM_REL
      ) as rel
      where rel.nr > 1;
  commit;
  select count(*) INTO v_numRecords 
      from
      (
          select 
              AGENT_ID, 
              VM_ID, 
              START_TIME, 
              END_TIME, 
              LAST_MODIFIED, 
              row_number() over ( PARTITION BY AGENT_ID, END_TIME) as nr 
          from adm.AGT_VM_REL
      ) as rel
      where rel.nr > 1;
  OPEN c1;
  WHILE (v_counter < v_numRecords) DO	
    FETCH c1 INTO V_AGENT_ID, V_VM_ID, V_START_TIME, V_END_TIME, V_LAST_MODIFIED;
    delete from adm.AGT_VM_REL where AGENT_ID = V_AGENT_ID and VM_ID = V_VM_ID and START_TIME = V_START_TIME 
      and END_TIME = V_END_TIME and LAST_MODIFIED = V_LAST_MODIFIED;
    IF MOD(v_counter, 1000) = 0 THEN
      commit;
    END IF;
   SET v_counter = v_counter + 1;
  END WHILE;
  CLOSE c1; 
  commit;
  end
%

call ADM.REMOVE_DUPLICATES ()%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.REMOVE_DUPLICATES')%

update adm.agent a set node_id =  (
    select v.node_id
    from adm.AGT_VM_REL rel
    join adm.vm v
        on v.id = rel.VM_ID
    where
        rel.AGENT_ID = a.id
        and rel.END_TIME = '9999-12-31 00:00:00.0'
)
where
    a.id in (
    select a.id from adm.AGT_VM_REL rel2
    join adm.vm v2 on rel2.vm_id = v2.id
    where rel2.AGENT_ID = a.id
      and rel2.END_TIME = '9999-12-31 00:00:00.0'
)%

call ADM.DROP_TABLE_IF_EXISTS('VIRT.GUEST_HIST')%

create table VIRT.GUEST_HIST
(
    ID                  bigint              not null,
    EXT_ID              varchar(100)	    ,
    TYPE				varchar(100)		not null,
    PARENT_ID          	bigint              not null,
    CPU_NUMBER			integer				,
    AGENT_ID			bigint				
)%

alter table VIRT.GUEST_HIST add constraint GUESTH_PK primary key
(
    ID
)%

grant select, insert, delete, update, alter on VIRT.GUEST_HIST to user TLMSRV%

call ADM.DROP_TABLE_IF_EXISTS('VIRT.HOST_HIST')%

create table VIRT.HOST_HIST
(
    ID                  bigint              not null,
    EXT_ID				varchar(100)		,
    TYPE				varchar(100)		not null,
    PARENT_ID          	bigint              not null,
    PARENT_TYPE        	integer		        not null,
    HOSTNAME			varchar(100)		,
    HARDWARE_VENDOR		varchar(100)		,
    HARDWARE_MODEL		varchar(100)		,
    PROC_VENDOR			varchar(100)		,
    PROC_DESCRIPTION	varchar(500)		,
    CPU_NUMBER			integer				,
    TOTAL_CORES 		integer				,
    CONNECTION_STATE	integer				,
	POWER_STATE			integer    
)%

alter table VIRT.HOST_HIST add constraint HOSTH_PK primary key
(
    ID
)%

grant select, insert, delete, update, alter on VIRT.HOST_HIST to user TLMSRV%

call ADM.DROP_PROCEDURE_IF_EXISTS('ADM.DROP_TABLE_IF_EXISTS')%	

drop procedure ADM.DROP_PROCEDURE_IF_EXISTS%