connect to tlma%

create procedure ADM.DROP_TRIGGER_IF_EXISTS(in TRIGGER_NAME char(100))
	begin
	declare STMT char(100);
	declare MSG char(20);
	
	declare continue handler for sqlstate '42704' set MSG = 'ok';
	
	set STMT = 'DROP TRIGGER ' concat TRIGGER_NAME;
	prepare S1 from STMT;
	execute S1;
	end%

ALTER TABLE ADM.AGENT_INV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%

ALTER TABLE ADM.SHARED_DISK_INV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%

ALTER TABLE ADM.PROD_INV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%

ALTER TABLE ADM.PROD_HINV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%

ALTER TABLE ADM.PVU_PROD_INV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%

ALTER TABLE ADM.STG_PROD_INV ADD COLUMN SIGNATURE_ID BIGINT NOT NULL DEFAULT -1%


call ADM.DROP_TRIGGER_IF_EXISTS('ADM.PROD_HINV_H1')%

create trigger ADM.PROD_HINV_H1
    after insert on ADM.PROD_INV
    referencing new as NEW for each row mode db2sql
    when (
        (NEW.TYPE <> 20) and
        ((select OS_NAME from ADM.AGENT where ID = NEW.AGENT_ID) <> 'i5/OS')
    )
    insert into ADM.PROD_HINV (
    	ID,
        SCOPE,
        SCOPE_ID,
        COMPONENT_ID,
		SIGNATURE_ID,
        PRODUCT_ID,
        AGENT_ID,
        BRANCH_ID,
        START_TIME,
        END_TIME)
    values (
        NEW.ID,
        NEW.SCOPE,
        NEW.SCOPE_ID,
        NEW.COMPONENT_ID,
        NEW.SIGNATURE_ID,
        NEW.PRODUCT_ID,
        NEW.AGENT_ID,
        NEW.BRANCH_ID,
        NEW.START_TIME,
        NEW.END_TIME)%
	
call ADM.DROP_TRIGGER_IF_EXISTS('ADM.PVU_PROD_INV_U')%

CREATE TRIGGER ADM.PVU_PROD_INV_U
	AFTER UPDATE OF
        scope,
        scope_id,
        component_id,
		SIGNATURE_ID,
        agent_id,
        branch_id,
        product_id,
        agent_inv_id,
        link_id,
        type,
        start_time,
        end_time,
        is_remote,
        last_modified
      ON adm.prod_inv  REFERENCING  NEW AS new  FOR EACH ROW  MODE DB2SQL
	BEGIN ATOMIC
        UPDATE adm.pvu_prod_inv ppi
            SET ppi.scope = new.scope, ppi.scope_id = new.scope_id,
                ppi.agent_id = new.agent_id, ppi.product_id = new.product_id,
                ppi.type = new.type, ppi.link_id = new.link_id, ppi.component_id = new.component_id,
				ppi.SIGNATURE_ID = new.SIGNATURE_ID,
                ppi.start_time = new.start_time, ppi.end_time = new.end_time,
                ppi.is_remote = new.is_remote, ppi.last_modified = new.last_modified
        WHERE new.id = ppi.id;

        UPDATE adm.stg_prod_inv spi
            SET spi.scope = new.scope, spi.scope_id = new.scope_id,
                spi.agent_id = new.agent_id, spi.product_id = new.product_id,
                spi.type = new.type, spi.link_id = new.link_id, spi.component_id = new.component_id,
				spi.SIGNATURE_ID = new.SIGNATURE_ID,
                spi.start_time = new.start_time, spi.end_time = new.end_time,
                spi.is_remote = new.is_remote, spi.last_modified = new.last_modified
        WHERE new.id = spi.id;		
	END%

call ADM.DROP_TRIGGER_IF_EXISTS('ADM.PVU_PROD_INV_I')%

CREATE TRIGGER ADM.PVU_PROD_INV_I
	AFTER INSERT
      ON adm.prod_inv
      REFERENCING  NEW AS n FOR EACH ROW  MODE DB2SQL
	BEGIN ATOMIC
        INSERT INTO adm.pvu_prod_inv (
            id, scope, scope_id, agent_id,
            component_id, SIGNATURE_ID,
            product_id, link_id, type,
            start_time,end_time,
            is_remote, last_modified)
        (
  			SELECT
                n.id, n.scope, n.scope_id, n.agent_id,
                n.component_id, n.SIGNATURE_ID,
                n.product_id, n.link_id, n.type,
                n.start_time, n.end_time,
                n.is_remote, n.last_modified
     		FROM
                swcat.swproduct sp
     		WHERE	
                    n.product_id = sp.id
                AND sp.tree_level = 3
                AND sp.is_pvu = 1
        );

        INSERT INTO adm.stg_prod_inv (
            id, scope, scope_id, agent_id,
            component_id, SIGNATURE_ID,
            product_id, link_id, type,
            start_time,end_time,
            is_remote, last_modified)
        (
  			SELECT
                n.id, n.scope, n.scope_id, n.agent_id,
                n.component_id, n.SIGNATURE_ID,
                n.product_id, n.link_id, n.type,
                n.start_time, n.end_time,
                n.is_remote, n.last_modified
     		FROM
                swcat.swproduct sp
     		WHERE	
                    n.product_id = sp.id
                AND sp.tree_level = 3
                AND sp.is_stg = 1
        );
	END%

create view CUSTOM.INSTALLED_OFFERING_SIGNATURE_DETAILS_V
as
with TEMP_SIGNATURE(ID,NAME,FILE_SIZE,PLATFORM,SIG_TYPE,SIG_SCOPE) as (
select ID,NAME,FILE_SIZE,PLATFORM,SIG_TYPE,SIG_SCOPE from SWCAT.SIGNATURE
union 
select -1,'NOT FOUND', -1, 'UNDEFINED', -1,-1 from SYSIBM.SYSDUMMY1
)
select	INV.ID as ID,
		INV.SCOPE as PATH,
		INV.AGENT_ID as AGENT_ID,
		INV.IS_REMOTE as IS_REMOTE,
		INV.START_TIME as START_TIME,
		INV.END_TIME as END_TIME,
		INV.PRODUCT_ID as RELEASE_ID,
		PROD.NAME as RELEASE_NAME,
		PROD.VERSION as RELEASE_VERSION,
		INV.COMPONENT_ID as COMPONENT_ID,
		COMP.NAME as COMPONENT_NAME,
		COMP.VERSION as COMPONENT_VERSION,
		COMP.PLATFORM as COMPONENT_PLATFORM,
		INV.SIGNATURE_ID as SIGNATURE_ID,
		SIG.NAME as SIGNATURE_NAME,
		SIG.FILE_SIZE as SIGNATURE_FILE_SIZE,
		SIG.PLATFORM as SIGNATURE_PLATFORM,
		SIG.SIG_TYPE as SIGNATURE_TYPE,
		SIG.SIG_SCOPE as SIGNATURE_SCOPE
from ADM.PROD_INV INV
inner join TEMP_SIGNATURE SIG on INV.SIGNATURE_ID = SIG.ID
inner join SWCAT.COMPONENT COMP on INV.COMPONENT_ID = COMP.ID
inner join SWCAT.SWPRODUCT PROD on INV.PRODUCT_ID = PROD.ID%

REORG TABLE ADM.AGENT_INV%

REORG TABLE ADM.SHARED_DISK_INV%

REORG TABLE ADM.PROD_INV%

REORG TABLE ADM.PROD_HINV%

REORG TABLE ADM.PVU_PROD_INV%

REORG TABLE ADM.STG_PROD_INV%

drop procedure ADM.DROP_TRIGGER_IF_EXISTS%

disconnect current%